@@ -11,8 +11,8 @@ from typing import TYPE_CHECKING, Any, Final, Iterable, final
 
 from bleak.backends.device import BLEDevice
 from bleak.backends.scanner import AdvertisementData
-from bluetooth_adapters import DiscoveredDeviceAdvertisementData, adapter_human_name
-from bluetooth_data_tools import monotonic_time_coarse
+from bluetooth_adapters import adapter_human_name
+from bluetooth_data_tools import monotonic_time_coarse, parse_advertisement_data_bytes
 
 from .central_manager import get_manager
 from .const import (
@@ -27,24 +27,18 @@ from .models import (
     HaBluetoothConnector,
     HaScannerDetails,
 )
+from .storage import DiscoveredDeviceAdvertisementData
 
 SCANNER_WATCHDOG_INTERVAL_SECONDS: Final = SCANNER_WATCHDOG_INTERVAL.total_seconds()
 _LOGGER = logging.getLogger(__name__)
 
 
+_bytes = bytes
 _float = float
 _int = int
 _str = str
 
 
-def _dict_subset(super_dict: dict[Any, bytes], sub_dict: dict[Any, bytes]) -> bool:
-    """Return True if sub_dict is a subset of super_dict."""
-    for key, sub_value in sub_dict.items():
-        if (super_value := super_dict.get(key)) is None or super_value != sub_value:
-            return False
-    return True
-
-
 class BaseHaScanner:
     """Base class for high availability BLE scanners."""
 
@@ -277,6 +271,7 @@ class BaseHaRemoteScanner(BaseHaScanner):
                 self.connectable,
                 discovered_device_timestamps[address],
                 adv.tx_power,
+                history.discovered_device_raw.get(address),
             )
             for address, (
                 device,
@@ -295,6 +290,7 @@ class BaseHaRemoteScanner(BaseHaScanner):
             self._expire_seconds,
             self._build_discovered_device_advertisement_datas(),
             self._build_discovered_device_timestamps(),
+            self._build_discovered_device_raw(),
         )
 
     @property
@@ -329,6 +325,12 @@ class BaseHaRemoteScanner(BaseHaScanner):
             address: info.time for address, info in self._previous_service_info.items()
         }
 
+    def _build_discovered_device_raw(self) -> dict[str, bytes | None]:
+        """Return a dict of discovered device raw advertisement data."""
+        return {
+            address: info.raw for address, info in self._previous_service_info.items()
+        }
+
     def _cancel_expire_devices(self) -> None:
         """Cancel the expiration of old devices."""
         if self._cancel_track:
@@ -399,6 +401,28 @@ class BaseHaRemoteScanner(BaseHaScanner):
             return info.device, info.advertisement
         return None
 
+    def _async_on_raw_advertisement(
+        self,
+        address: _str,
+        rssi: _int,
+        raw: _bytes,
+        details: dict[str, Any],
+        advertisement_monotonic_time: _float,
+    ) -> None:
+        parsed = parse_advertisement_data_bytes(raw)
+        self._async_on_advertisement_internal(
+            address,
+            rssi,
+            parsed[0],
+            parsed[1],
+            parsed[2],
+            parsed[3],
+            parsed[4],
+            details,
+            advertisement_monotonic_time,
+            raw,
+        )
+
     def _async_on_advertisement(
         self,
         address: _str,
@@ -410,6 +434,32 @@ class BaseHaRemoteScanner(BaseHaScanner):
         tx_power: _int | None,
         details: dict[Any, Any],
         advertisement_monotonic_time: _float,
+    ) -> None:
+        self._async_on_advertisement_internal(
+            address,
+            rssi,
+            local_name,
+            service_uuids,
+            service_data,
+            manufacturer_data,
+            tx_power,
+            details,
+            advertisement_monotonic_time,
+            None,
+        )
+
+    def _async_on_advertisement_internal(
+        self,
+        address: _str,
+        rssi: _int,
+        local_name: _str | None,
+        service_uuids: list[str],
+        service_data: dict[str, bytes],
+        manufacturer_data: dict[int, bytes],
+        tx_power: _int | None,
+        details: dict[Any, Any],
+        advertisement_monotonic_time: _float,
+        raw: _bytes | None,
     ) -> None:
         """Call the registered callback."""
         self.scanning = not self._connecting
@@ -449,15 +499,15 @@ class BaseHaRemoteScanner(BaseHaScanner):
             # _rssi is deprecated, will be removed in newer bleak
             # pylint: disable-next=protected-access
             info.device._rssi = rssi
-            has_local_name = bool(local_name)
-            if prev_name and (
-                not has_local_name
-                or (prev_name is not local_name and len(prev_name) > len(local_name))  # type: ignore[arg-type]
+            if prev_name is not None and (
+                prev_name is local_name
+                or not local_name
+                or len(prev_name) > len(local_name)
             ):
                 info.name = prev_name
             else:
                 info.device.name = local_name
-                info.name = local_name if has_local_name else address  # type: ignore[assignment]
+                info.name = local_name if local_name else address
 
             has_service_uuids = bool(service_uuids)
             if (
@@ -473,13 +523,17 @@ class BaseHaRemoteScanner(BaseHaScanner):
 
             has_service_data = bool(service_data)
             if has_service_data and service_data is not prev_info.service_data:
-                if _dict_subset(prev_info.service_data, service_data):
-                    info.service_data = prev_info.service_data
+                for uuid, sub_value in service_data.items():
+                    if (
+                        super_value := prev_info.service_data.get(uuid)
+                    ) is None or super_value != sub_value:
+                        info.service_data = {
+                            **prev_info.service_data,
+                            **service_data,
+                        }
+                        break
                 else:
-                    info.service_data = {
-                        **prev_info.service_data,
-                        **service_data,
-                    }
+                    info.service_data = prev_info.service_data
             elif not has_service_data:
                 info.service_data = prev_info.service_data
             else:
@@ -490,13 +544,17 @@ class BaseHaRemoteScanner(BaseHaScanner):
                 has_manufacturer_data
                 and manufacturer_data is not prev_info.manufacturer_data
             ):
-                if _dict_subset(prev_info.manufacturer_data, manufacturer_data):
-                    info.manufacturer_data = prev_info.manufacturer_data
+                for id_, sub_value in manufacturer_data.items():
+                    if (
+                        super_value := prev_info.manufacturer_data.get(id_)
+                    ) is None or super_value != sub_value:
+                        info.manufacturer_data = {
+                            **prev_info.manufacturer_data,
+                            **manufacturer_data,
+                        }
+                        break
                 else:
-                    info.manufacturer_data = {
-                        **prev_info.manufacturer_data,
-                        **manufacturer_data,
-                    }
+                    info.manufacturer_data = prev_info.manufacturer_data
             elif not has_manufacturer_data:
                 info.manufacturer_data = prev_info.manufacturer_data
             else:
@@ -509,6 +567,7 @@ class BaseHaRemoteScanner(BaseHaScanner):
         info.connectable = self.connectable
         info.time = advertisement_monotonic_time
         info.tx_power = tx_power
+        info.raw = raw
         self._previous_service_info[address] = info
         self._manager.scanner_adv_received(info)
 
