@@ -5,6 +5,7 @@ from __future__ import annotations
 import asyncio
 import time
 from datetime import timedelta
+from unittest.mock import ANY
 
 import pytest
 from bleak.backends.device import BLEDevice
@@ -24,6 +25,9 @@ from habluetooth.const import (
     SCANNER_WATCHDOG_INTERVAL,
     SCANNER_WATCHDOG_TIMEOUT,
 )
+from habluetooth.storage import (
+    DiscoveredDeviceAdvertisementData,
+)
 
 from . import (
     MockBleakClient,
@@ -57,6 +61,22 @@ class FakeScanner(BaseHaRemoteScanner):
             now or monotonic_time_coarse(),
         )
 
+    def inject_raw_advertisement(
+        self,
+        address: str,
+        rssi: int,
+        adv: bytes,
+        now: float | None = None,
+    ) -> None:
+        """Inject a raw advertisement."""
+        self._async_on_raw_advertisement(
+            address,
+            rssi,
+            adv,
+            {"scanner_specific_data": "test"},
+            now or monotonic_time_coarse(),
+        )
+
 
 @pytest.mark.parametrize("name_2", [None, "w"])
 @pytest.mark.usefixtures("enable_bluetooth")
@@ -177,6 +197,40 @@ async def test_remote_scanner(name_2: str | None) -> None:
         switchbot_device_2.address
     ][1].manufacturer_data == {1: b"\x04", 2: b"\x01", 3: b"\x03"}
 
+    assert (
+        "00090401-0052-036b-3206-ff0a050a021a"
+        not in scanner.discovered_devices_and_advertisement_data[
+            switchbot_device_2.address
+        ][1].service_data
+    )
+
+    scanner.inject_raw_advertisement(
+        switchbot_device_2.address,
+        switchbot_device_2.rssi,
+        b"\x12\x21\x1a\x02\n\x05\n\xff\x062k\x03R\x00\x01\x04\t\x00\x04",
+    )
+
+    assert (
+        "00090401-0052-036b-3206-ff0a050a021a"
+        in scanner.discovered_devices_and_advertisement_data[
+            switchbot_device_2.address
+        ][1].service_data
+    )
+
+    assert scanner.serialize_discovered_devices() == DiscoveredDeviceAdvertisementData(
+        connectable=True,
+        expire_seconds=195,
+        discovered_device_advertisement_datas={"44:44:33:11:23:45": ANY},
+        discovered_device_timestamps={"44:44:33:11:23:45": ANY},
+        discovered_device_raw={
+            "44:44:33:11:23:45": b"\x12!\x1a\x02"
+            b"\n\x05\n\xff"
+            b"\x062k\x03"
+            b"R\x00\x01\x04"
+            b"\t\x00\x04"
+        },
+    )
+
     cancel()
     unsetup()
 
@@ -593,3 +647,50 @@ async def test_merge_manufacturer_data_history_new() -> None:
 
     cancel()
     unsetup()
+
+
+@pytest.mark.usefixtures("enable_bluetooth")
+@pytest.mark.asyncio
+async def test_filter_apple_data() -> None:
+    """Test filtering apple data accepts bytes that start with 01."""
+    manager = get_manager()
+
+    device = generate_ble_device(
+        "44:44:33:11:23:45",
+        "",
+        {},
+        rssi=-60,
+    )
+    device_adv = generate_advertisement_data(
+        local_name="",
+        rssi=-60,
+        manufacturer_data={
+            76: b"\x01\r.\xa9\xb6",
+        },
+        service_uuids=["ef090000-11d6-42ba-93b8-9dd7ec090ab0"],
+        service_data={},
+    )
+
+    connector = HaBluetoothConnector(
+        MockBleakClient, "mock_bleak_client", lambda: False
+    )
+    scanner = FakeScanner("esp32", "esp32", connector, True)
+    details = scanner.details
+    assert details == HaScannerDetails(
+        source=scanner.source,
+        connectable=scanner.connectable,
+        name=scanner.name,
+        adapter=scanner.adapter,
+    )
+    unsetup = scanner.async_setup()
+    cancel = manager.async_register_scanner(scanner)
+
+    scanner.inject_advertisement(device, device_adv)
+
+    data = scanner.discovered_devices_and_advertisement_data
+    discovered_device, discovered_adv_data = data[device.address]
+    assert discovered_device.address == device.address
+    assert discovered_device.name == device.name
+    assert discovered_adv_data.manufacturer_data == device_adv.manufacturer_data
+    unsetup()
+    cancel()
